@@ -26,22 +26,6 @@ def simple_config():
     return yaml.safe_load(config)
 
 
-@pytest.fixture
-def simple_data():
-    return {
-        "album": [
-            {
-                "id": "None_0",
-                "name": "Five Leaves Left",
-                "artist": "Nick Drake",
-                "released": "1969",
-                "label": "Island",
-                "genre": "Folk",
-            }
-        ]
-    }
-
-
 @pytest.fixture
 def debug_logger():
     logger = logging.getLogger("test")
